import {Injectable, OnDestroy} from '@angular/core'
import {Observable, Subject} from 'rxjs'
import {take, takeUntil} from 'rxjs/operators'
import {AppState} from '../../shared/app-state'
import {HttpClient, HttpResponse} from '@angular/common/http'
import {environment} from '../../../../environments/environment'
import {ApiService} from '../../shared/service/api.service'
import {Permission, User} from '../../shared/model/user'
import {ErrorService} from '../../shared/service/error.service'
import {AlertService} from '../../shared/service/alert.service'
import {JwtService} from "./jwt.service";

@Injectable({
  providedIn: 'root'
})
export class AccountService implements OnDestroy {
  private destroy$ = new Subject<boolean>()

  constructor(
    private http: HttpClient,
    private api: ApiService,
    private appState: AppState,
    private jwtService: JwtService,
    private errorService: ErrorService,
    private alertService: AlertService
  ) {
  }

  ngOnDestroy(): void {
    this.destroy$.next(true)
    this.destroy$.complete()
  }

  checkAuthenticationStatus() {
    if (!this.appState.isAuthenticated) {
      // Try to get current default group user
      this.http.get<User>(`${environment.apiUrl}/user/group/currentuser`, {withCredentials: true})
        .pipe(
          take(1),
          takeUntil(this.destroy$),
        ).subscribe(
        {
          next: user => this.appState.authenticateGroupUser(user),
          error: err => {
            if (err.status === 404 || err.status === 403) {
              console.warn('No default user is defined on this computer')
            } else {
              this.errorService.handleError(err)
            }
          }
        })
    }
  }

  login(userId: number, password: string): Observable<any> {
    const subject = new Subject<void>()

    this.http.post<any>(`${environment.apiUrl}/login`, {id: userId, password}, {
      withCredentials: true,
      observe: 'response' as 'body'
    }).pipe(
      take(1),
      takeUntil(this.destroy$)
    ).subscribe({
      next: (response: HttpResponse<void>) => {
        this.loginUser(response)

        subject.next()
        subject.complete()
      },
      error: error => {
        if (error.status === 403) {
          this.alertService.pushError('Les identifiants entrés sont invalides')
        } else {
          this.errorService.handleError(error)
        }

        subject.next()
        subject.complete()
      }
    })

    return subject
  }

  private loginUser(response: HttpResponse<void>) {
    const authorization = response.headers.get("Authorization")
    const user = this.jwtService.parseUser(authorization)

    this.appState.authenticateUser(user)
  }

  logout(): Observable<void> {
    const subject = new Subject<void>()

    this.api.get<void>('/logout').pipe(
      take(1),
      takeUntil(this.destroy$)
    ).subscribe({
      next: () => {
        this.logoutUser()

        subject.next()
        subject.complete()
      },
      error: error => {
        this.errorService.handleError(error)

        subject.next()
        subject.complete()
      }
    })

    return subject
  }

  private logoutUser() {
    this.appState.resetAuthenticatedUser()
    this.checkAuthenticationStatus()
  }

  hasPermission(permission: Permission): boolean {
    return this.appState.authenticatedUser && this.appState.authenticatedUser.permissions.indexOf(permission) >= 0
  }
}
