import {
  AfterViewInit,
  Component,
  ContentChild,
  Directive,
  EventEmitter,
  Input,
  Output,
  ViewChild,
  ViewEncapsulation
} from '@angular/core'
import {ConfigService} from '../shared/service/config.service'
import {Config, ConfigControl} from '../shared/model/config.model'
import {SubscribingComponent} from '../shared/components/subscribing.component'
import {ErrorService} from '../shared/service/error.service'
import {ActivatedRoute, Router} from '@angular/router'
import {formatDate, formatDateTime, getConfiguredImageUrl, getFileUrl, readFile} from '../shared/utils/utils'
import {AbstractControl} from '@angular/forms'
import {CrePromptDialog} from '../shared/components/dialogs/dialogs'

@Directive({
  selector: 'cre-config-list'
})
export class CreConfigList {
}

@Component({
  selector: 'cre-config-actions',
  templateUrl: 'config-actions.html'
})
export class CreConfigActions {
}

@Component({
  selector: 'cre-config-section',
  templateUrl: 'config-section.html'
})
export class CreConfigSection {
  @Input() label: string

  @ContentChild(CreConfigActions) actions: CreConfigActions

  get hasActions(): boolean {
    return this.actions !== undefined
  }
}

@Component({
  selector: 'cre-config-container',
  templateUrl: 'config-container.html',
  styleUrls: ['config.sass'],
  encapsulation: ViewEncapsulation.None
})
export class CreConfigContainer {
  @Input() configuration?: Config
  @Input() tooltip: string

  get readOnly(): boolean {
    return !this.configuration?.editable ?? true
  }
}

@Directive()
abstract class _CreConfigBase extends SubscribingComponent {
  @Input() configControl: ConfigControl
  @Input() label: string
  @Input() tooltip?: string

  constructor(
    private configService: ConfigService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
  }

  get config(): Config {
    return this.configControl.config
  }

  get control(): AbstractControl {
    return this.configControl.control
  }

  get lastUpdated(): string {
    return 'Dernière mise à jour: ' + formatDateTime(this.config.lastUpdated)
  }

  get inputHint(): string {
    return this.config?.editable ? this.lastUpdated : null
  }
}

@Directive()
abstract class _CreTextConfigBase extends _CreConfigBase {
  private static readonly REQUIRE_RESTART_ICON = 'alert'
  private static readonly REQUIRE_RESTART_ICON_TITLE = 'Requiert un redémarrage'

  get inputIcon(): string {
    return this.config?.requireRestart ? _CreTextConfigBase.REQUIRE_RESTART_ICON : null
  }

  get inputIconTitle(): string {
    return this.config?.requireRestart ? _CreTextConfigBase.REQUIRE_RESTART_ICON_TITLE : null
  }
}

@Component({
  selector: 'cre-text-config',
  templateUrl: 'config-text.html',
  styleUrls: ['config.sass']
})
export class CreTextConfig extends _CreTextConfigBase {
}

@Component({
  selector: 'cre-image-config',
  templateUrl: 'config-image.html',
  styleUrls: ['config.sass'],
  encapsulation: ViewEncapsulation.None
})
export class CreImageConfig extends _CreConfigBase {
  @Input() previewWidth: string | null

  @Output() invalidFormat = new EventEmitter<void>()

  updatedImage: any | null

  updateImage(file: File): any {
    readFile(file, (content) => this.updatedImage = content)
  }

  get imageUrl(): string {
    const path = this.config.key == Config.INSTANCE_ICON_SET ? 'icon' : 'logo'
    return getConfiguredImageUrl(path)
  }
}

@Component({
  selector: 'cre-bool-config',
  templateUrl: 'config-bool.html'
})
export class CreBoolConfig extends _CreConfigBase {
}

@Component({
  selector: 'cre-period-config',
  templateUrl: 'config-period.html'
})
export class CrePeriodConfig extends _CreConfigBase {
}

@Component({
  selector: 'cre-date-config',
  templateUrl: 'config-date.html'
})
export class CreDateConfig extends _CreTextConfigBase implements AfterViewInit {
  ngAfterViewInit(): void {
    this.control.setValue(formatDate(this.config.content))
  }
}

@Component({
  selector: 'cre-secure-config',
  templateUrl: 'config-secure.html'
})
export class CreSecureConfig extends _CreTextConfigBase {
  @ViewChild(CrePromptDialog) dialog: CrePromptDialog

  @Input() buttonLabel: string

  private initialValue: string | null

  constructor(
    configService: ConfigService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(configService, errorService, activatedRoute, router)
  }

  onOpen() {
    this.initialValue = this.control.value
    this.dialog.show()
  }

  onCancel() {
    this.control.setValue(this.initialValue)
  }
}
