import {Component} from '@angular/core'
import {MaterialType} from '../../../shared/model/materialtype.model'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {MaterialTypeService} from '../../service/material-type.service'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {Validators} from '@angular/forms'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.sass']
})
export class EditComponent extends ErrorHandlingComponent {
  materialType: MaterialType | null
  formFields: FormField[] = [
    {
      name: 'name',
      label: 'Nom',
      icon: 'form-textbox',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: (errors) => errors.required, message: 'Un nom est requis'},
      ]
    },
    {
      name: 'prefix',
      label: 'Préfixe',
      icon: 'label-variant',
      type: 'text',
      required: true,
      validator: Validators.compose([Validators.minLength(3), Validators.maxLength(3)]),
      errorMessages: [
        {conditionFn: (errors) => errors.required, message: 'Un préfixe est requis'},
        {
          conditionFn: (errors) => errors.minlength || errors.maxlength,
          message: 'Le préfixe doit faire exactement 3 caractères'
        }
      ]
    }
  ]

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'notfound-materialtype-id',
    consumer: error => this.urlUtils.navigateTo('/catalog/materialtype/list')
  }, {
    filter: error => error.type === 'exists-materialtype-name',
    messageProducer: error => `Un type de produit avec le nom '${error.name}' existe déjà`
  }, {
    filter: error => error.type === 'exists-materialtype-prefix',
    messageProducer: error => `Un type de produit avec le préfixe '${error.prefix}' existe déjà`
  }, {
    filter: error => error.type === 'cannotdelete-materialtype',
    messageProducer: error => 'Ce type de produit est utilisé dans une ou plusieurs recettes ou produits'
  }]

  constructor(
    private materialTypeService: MaterialTypeService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    const id = parseInt(this.activatedRoute.snapshot.paramMap.get('id'))
    this.subscribeEntityById(
      this.materialTypeService,
      id,
      materialType => {
        this.appState.title = `${materialType.name} (Modifications)`
        this.materialType = materialType
      }
    )
  }

  submit(values) {
    this.subscribeAndNavigate(
      this.materialTypeService.update(this.materialType.id, values.name, values.prefix),
      '/catalog/materialtype/list'
    )
  }

  delete() {
    this.subscribeAndNavigate(
      this.materialTypeService.delete(this.materialType.id),
      '/catalog/materialtype/list'
    )
  }
}
