import {ChangeDetectorRef, Component} from '@angular/core'
import {ErrorHandlingComponent} from '../shared/components/subscribing.component'
import {Company} from '../shared/model/company.model'
import {getRecipeLuma, Recipe, recipeMatchesFilter} from '../shared/model/recipe.model'
import {CompanyService} from '../company/service/company.service'
import {RecipeService} from './services/recipe.service'
import {AccountService} from '../accounts/services/account.service'
import {ConfigService} from '../shared/service/config.service'
import {AppState} from '../shared/app-state'
import {ErrorService} from '../shared/service/error.service'
import {ActivatedRoute, Router} from '@angular/router'
import {Config} from '../shared/model/config.model'
import {Permission} from '../shared/model/user'
import {FormControl} from '@angular/forms'

@Component({
  selector: 'cre-recipe-list',
  templateUrl: 'list.html',
  styleUrls: ['recipes.sass']
})
export class RecipeList extends ErrorHandlingComponent {
  companies: Company[] = []
  recipes: Map<number, Recipe[]> = new Map<number, Recipe[]>()
  columns = ['name', 'description', 'color', 'sample', 'iconNotApproved', 'buttonView', 'buttonEdit']
  panelForcedExpanded = false

  searchControl: FormControl
  searchQuery = ''

  recipeFilterPredicate = recipeMatchesFilter

  constructor(
    private companyService: CompanyService,
    private recipeService: RecipeService,
    private accountService: AccountService,
    private configService: ConfigService,
    private cdRef: ChangeDetectorRef,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()
    this.appState.title = 'Explorateur'

    // Navigate to configs if server is in emergency mode
    this.subscribe(
      this.configService.get(Config.EMERGENCY_MODE),
      config => {
        if (config.content == 'true') {
          this.urlUtils.navigateTo('/admin/config/')
        }
      }
    )

    this.fetchCompanies()
    this.fetchRecipes()

    this.searchControl = new FormControl('')
    this.subscribe(
      this.searchControl.valueChanges,
      value => {
        this.searchQuery = value
        if (value.length > 0 && !this.panelForcedExpanded) {
          this.panelForcedExpanded = true
          this.cdRef.detectChanges()
        }
      }
    )
  }

  private fetchCompanies() {
    this.subscribe(
      this.companyService.all,
      companies => this.companies = companies
    )
  }

  private fetchRecipes() {
    this.subscribe(
      this.recipeService.allByCompany,
      recipes => this.recipes = recipes,
      true
    )
  }

  isCompanyHidden(company: Company): boolean {
    const companyRecipes = this.recipes.get(company.id)
    return !(companyRecipes && companyRecipes.length >= 0) ||
      this.searchQuery && this.searchQuery.length > 0 &&
      !companyRecipes.some(recipe => this.recipeFilterPredicate(recipe, this.searchQuery))
  }


  isLight(recipe: Recipe): boolean {
    return getRecipeLuma(recipe) > 200
  }

  get hasEditPermission(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_RECIPES)
  }

  get hasCompanyEditPermission(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_COMPANIES)
  }
}
