import {AfterViewInit, ChangeDetectorRef, Component, Input, OnDestroy, OnInit, ViewChild, ViewChildren} from '@angular/core'
import {CreTable} from '../../shared/components/tables/tables'
import {Mix, MixMaterialDto, mixMaterialsToMixMaterialsDto, sortMixMaterialsDto} from '../../shared/model/recipe.model'
import {Observable, Subject} from 'rxjs'
import {Material, materialComparator} from '../../shared/model/material.model'
import {FormControl, Validators} from '@angular/forms'
import {takeUntil} from 'rxjs/operators'
import {CreComboBoxComponent, CreInputEntry} from '../../shared/components/inputs/inputs'
import {AccountService} from '../../accounts/services/account.service'
import {Permission} from '../../shared/model/user'
import {UNIT_MILLILITER} from '../../shared/units'

@Component({
  selector: 'cre-mix-materials-form-combo-box',
  templateUrl: 'materials-form-combo-box.html'
})
export class MixMaterialsFormComboBox implements OnInit {
  @ViewChild(CreComboBoxComponent) comboBox: CreComboBoxComponent

  @Input() mixMaterial: MixMaterialDto
  @Input() mix: Mix | null
  @Input() mixMaterials: MixMaterialDto[]
  @Input() control: FormControl
  @Input() materials: Material[]
  @Input() position: number

  entries: CreInputEntry[]

  ngOnInit() {
    this.entries = this.filterMaterials()
  }

  updateEntries() {
    this.entries = this.filterMaterials()
    this.comboBox.reloadEntries()
  }

  private filterMaterials(): CreInputEntry[] {
    return this.materials
      .filter(material => {
        if (this.mix && this.mix.mixType.material.id === material.id) {
          return false
        }

        // Prevent use of percents in first position
        if (material.materialType.usePercentages && this.mixMaterial.position <= 1) {
          return false
        }

        if (this.mixMaterial.materialId === material.id) {
          return true
        }

        return this.mixMaterials.filter(x => x.materialId === material.id).length <= 0
      })
      .sort(materialComparator)
      .map(material => new CreInputEntry(material.id, material.name, material.materialType.prefix ? `[${material.materialType.prefix}] ${material.name}` : material.name))
  }
}

@Component({
  selector: 'cre-mix-materials-form',
  templateUrl: 'materials-form.html'
})
export class MixMaterialsForm implements AfterViewInit, OnDestroy {
  @ViewChild(CreTable) table: CreTable<MixMaterialDto>
  @ViewChildren(MixMaterialsFormComboBox) comboBoxes: MixMaterialsFormComboBox[]

  @Input() materials: Observable<Material[]>
  @Input() mix: Mix | null

  mixMaterials: MixMaterialDto[] = []
  columns = ['position', 'positionButtons', 'material', 'quantity', 'units', 'endButton']
  allMaterials: Material[]

  private _controls: ControlsByPosition[] = []
  private _destroy$ = new Subject<boolean>()

  constructor(
    private accountService: AccountService,
    private cdRef: ChangeDetectorRef
  ) {
  }

  ngAfterViewInit() {
    this.materials.subscribe({
      next: materials => {
        this.allMaterials = materials

        if (!this.mix) {
          this.addRow()
        } else {
          mixMaterialsToMixMaterialsDto(this.mix).forEach(x => this.insertRow(x))
        }

        this.table.renderRows()
        this.cdRef.detectChanges()
      }
    })
  }

  ngOnDestroy() {
    this._destroy$.next(true)
    this._destroy$.complete()
  }

  addRow() {
    const mixMaterial = new MixMaterialDto(null, 0, false, this.nextPosition, UNIT_MILLILITER)
    this.insertRow(mixMaterial)
    this.table.renderRows()
  }

  insertRow(mixMaterial: MixMaterialDto) {
    const materialIdControl = new FormControl(mixMaterial.materialId, Validators.required)
    const quantityControl = new FormControl(mixMaterial.quantity, Validators.required)
    const unitsControl = new FormControl(mixMaterial.units, Validators.required)

    materialIdControl.valueChanges
      .pipe(takeUntil(this._destroy$))
      .subscribe({
        next: materialId => {
          mixMaterial.materialId = materialId
          this.refreshAvailableMaterials()
          this.cdRef.detectChanges()
        }
      })

    this.mixMaterials.push(mixMaterial)
    this._controls.push({
      position: mixMaterial.position,
      controls: {
        materialId: materialIdControl,
        quantity: quantityControl,
        units: unitsControl
      }
    })
  }

  removeRow(mixMaterial: MixMaterialDto) {
    this.mixMaterials = this.mixMaterials.filter(x => x.position !== mixMaterial.position)
    this._controls = this._controls.filter(x => x.position !== mixMaterial.position)

    for (let position = mixMaterial.position + 1; position < this.mixMaterials.length; position++) {
      this.updatePosition(this.getMixMaterialByPosition(position), position - 1, false)
    }
  }

  updatePosition(mixMaterial: MixMaterialDto, newPosition: number, switchPositions = true) {
    const currentPosition = mixMaterial.position
    const currentControls = this.getControlsByPosition(currentPosition)

    // Update before current to prevent position conflicts
    if (switchPositions) {
      this.updatePosition(this.getMixMaterialByPosition(newPosition), currentPosition, false)
    }

    mixMaterial.position = newPosition
    currentControls.position = newPosition

    this.sortTable()
    this.refreshAvailableMaterials()
  }

  getControls(position: number): MixMaterialControls {
    return this.getControlsByPosition(position).controls
  }

  areUnitsPercents(mixMaterial: MixMaterialDto): boolean {
    if (!mixMaterial) {
      return false
    }

    return mixMaterial.materialId ? this.allMaterials?.filter(x => x.id === mixMaterial.materialId)[0].materialType.usePercentages : false
  }

  isDecreasePositionButtonDisabled(mixMaterial: MixMaterialDto): boolean {
    return mixMaterial.position <= 2 && this.areUnitsPercents(mixMaterial)
  }

  isIncreasePositionButtonDisabled(mixMaterial: MixMaterialDto): boolean {
    if (mixMaterial.position === this.mixMaterials.length) {
      return true
    }

    if (mixMaterial.position > 1) {
      return false
    }

    const nextMixMaterial = this.getMixMaterialByPosition(mixMaterial.position + 1)
    return this.areUnitsPercents(nextMixMaterial)
  }

  get hasMaterialEditPermission(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_MATERIALS)
  }

  get materialCount(): number {
    return this.allMaterials ? this.allMaterials.length : 0
  }

  get updatedMixMaterials(): MixMaterialDto[] {
    const updatedMixMaterials: MixMaterialDto[] = []
    this.mixMaterials.forEach(mixMaterial => {
      const controls = this.getControlsByPosition(mixMaterial.position).controls
      updatedMixMaterials.push({
        materialId: controls.materialId.value,
        quantity: controls.quantity.value,
        position: mixMaterial.position,
        units: controls.units.value,
        isPercents: this.areUnitsPercents(mixMaterial)
      })
    })
    return updatedMixMaterials
  }

  get valid(): boolean {
    return this._controls
      .map(controls => controls.controls)
      .map(controls => [controls.materialId, controls.quantity])
      .flatMap(controls => controls)
      .every(control => control.valid)
  }

  private get nextPosition(): number {
    return this.mixMaterials.length + 1
  }

  private getMixMaterialByPosition(position: number): MixMaterialDto {
    return this.mixMaterials.filter(x => x.position === position)[0]
  }

  private getControlsByPosition(position: number): ControlsByPosition {
    return this._controls.filter(control => control.position === position)[0]
  }

  private refreshAvailableMaterials() {
    this.comboBoxes.forEach(x => x.updateEntries())
  }

  private sortTable() {
    this.mixMaterials = sortMixMaterialsDto(this.mixMaterials)
    this.table.renderRows()
  }
}

interface MixMaterialControls {
  materialId: FormControl
  quantity: FormControl
  units: FormControl
}

interface ControlsByPosition {
  position: number
  controls: MixMaterialControls
}
