import {Component, Directive, Input, OnInit, ViewChild} from '@angular/core'
import {SubscribingComponent} from '../../shared/components/subscribing.component'
import {Mix, Recipe} from '../../shared/model/recipe.model'
import {ErrorService} from '../../shared/service/error.service'
import {ActivatedRoute, Router} from '@angular/router'
import {RecipeService} from '../services/recipe.service'
import {FormControl, Validators} from '@angular/forms'
import {Observable} from 'rxjs'
import {MaterialType} from '../../shared/model/materialtype.model'
import {MaterialTypeService} from '../../material-type/service/material-type.service'
import {CreInputEntry} from '../../shared/components/inputs/inputs'
import {map} from 'rxjs/operators'
import {Material} from '../../shared/model/material.model'
import {MaterialService} from '../../material/service/material.service'
import {CreForm} from '../../shared/components/forms/forms'
import {MixMaterialsForm} from './materials-form'
import {MixSaveDto, MixService, MixUpdateDto} from '../services/mix.service'

@Directive()
abstract class _BaseMixPage extends SubscribingComponent {
  materialTypes$ = this.materialTypeService.all
  materials$: Observable<Material[]>

  private _recipe: Recipe | null

  constructor(
    protected mixService: MixService,
    private recipeService: RecipeService,
    private materialTypeService: MaterialTypeService,
    protected materialService: MaterialService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    this.fetchRecipe()
  }

  private fetchRecipe() {
    const recipeId = this.urlUtils.parseIntUrlParam('recipeId')

    this.subscribe(
      this.recipeService.getById(recipeId),
      recipe => this.recipe = recipe
    )
  }

  set recipe(recipe: Recipe) {
    this._recipe = recipe
    this.materials$ = this.fetchMaterials(recipe.id)
  }

  get recipe(): Recipe {
    return this._recipe
  }

  protected abstract fetchMaterials(recipeId: number): Observable<Material[]>

  abstract submit(dto: MixSaveDto)
}

@Component({
  selector: 'cre-mix-add',
  templateUrl: 'add.html'
})
export class MixAdd extends _BaseMixPage {
  protected fetchMaterials(recipeId: number): Observable<Material[]> {
    return this.materialService.getAllForMixCreation(recipeId)
  }

  submit(dto: MixSaveDto) {
    this.subscribeAndNavigate(
      this.mixService.saveDto(dto),
      `/color/edit/${this.recipe.id}`
    )
  }
}

@Component({
  selector: 'cre-mix-edit',
  templateUrl: 'edit.html'
})
export class MixEdit extends _BaseMixPage {
  mix: Mix

  private mixId: number

  ngOnInit() {
    super.ngOnInit()

    this.mixId = this.urlUtils.parseIntUrlParam('id')

    this.fetchMix()
  }

  private fetchMix() {
    this.subscribe(
      this.mixService.getById(this.mixId),
      mix => this.mix = mix
    )
  }

  protected fetchMaterials(recipeId: number): Observable<Material[]> {
    return this.materialService.getAllForMixUpdate(this.mixId)
  }

  submit(dto: MixSaveDto) {
    this.subscribeAndNavigate(
      this.mixService.updateDto({...dto, id: this.mix.id}),
      `/color/edit/${this.recipe.id}`
    )
  }
}

@Component({
  selector: 'cre-mix-info-form',
  templateUrl: 'info-form.html'
})
export class MixInfoForm implements OnInit {
  @ViewChild(CreForm) form: CreForm

  @Input() recipe: Recipe
  @Input() mix: Mix | null
  @Input() materialTypes: Observable<MaterialType[]>

  materialTypeEntries: Observable<CreInputEntry[]>
  controls: any

  ngOnInit() {
    this.materialTypeEntries = this.materialTypes.pipe(
      map(materialTypes => {
        return materialTypes.map(materialType => new CreInputEntry(materialType.id, materialType.name))
      })
    )

    this.controls = {
      name: new FormControl(this.mix?.mixType.name, Validators.required),
      materialType: new FormControl(this.mix?.mixType.material.materialType.id, Validators.required)
    }
  }

  get mixName(): string {
    return this.controls.name.value
  }

  get mixMaterialTypeId(): number {
    return this.controls.materialType.value
  }

  get valid(): boolean {
    return this.form.valid
  }
}

@Component({
  selector: 'cre-mix-form',
  templateUrl: 'form.html'
})
export class MixForm {
  @ViewChild(MixInfoForm) infoForm: MixInfoForm
  @ViewChild(MixMaterialsForm) mixMaterialsForm: MixMaterialsForm

  @Input() recipe: Recipe
  @Input() mix: Mix | null
  @Input() materialTypes: Observable<MaterialType[]>
  @Input() materials: Observable<Material[]>

  get formValues(): MixSaveDto {
    return {
      name: this.infoForm.mixName,
      recipeId: this.recipe.id,
      materialTypeId: this.infoForm.mixMaterialTypeId,
      mixMaterials: this.mixMaterialsForm.updatedMixMaterials
    }
  }

  get valid(): boolean {
    return this.infoForm?.valid && this.mixMaterialsForm?.valid
  }
}
