import {ErrorHandlingComponent, SubscribingComponent} from '../shared/components/subscribing.component';
import {Observable, Subject} from 'rxjs';
import {CreInputEntry} from '../shared/components/inputs/inputs';
import {map, tap} from 'rxjs/operators';
import {RecipeService} from './services/recipe.service';
import {CompanyService} from '../company/service/company.service';
import {AppState} from '../shared/app-state';
import {ErrorHandler, ErrorService} from '../shared/service/error.service';
import {ActivatedRoute, Router} from '@angular/router';
import {FormControl, Validators} from '@angular/forms';
import {Component, EventEmitter, Input, Output, ViewChild, ViewEncapsulation} from '@angular/core';
import {Recipe, recipeMixCount, RecipeStep, recipeStepCount} from '../shared/model/recipe.model';
import {AccountService} from '../accounts/services/account.service';
import {Permission} from '../shared/model/user';
import {AlertService} from '../shared/service/alert.service';
import {GroupService} from '../groups/services/group.service';
import {StepTableComponent} from './components/step-table/step-table.component';
import {anyMap} from '../shared/utils/map.utils';
import {CreForm, ICreForm} from '../shared/components/forms/forms';

@Component({
  selector: 'recipe-form',
  templateUrl: 'form.html',
  styleUrls: ['recipes.sass'],
  encapsulation: ViewEncapsulation.None
})
export class RecipeForm extends SubscribingComponent {
  @ViewChild(CreForm) creForm: ICreForm

  @Input() recipe: Recipe | null

  @Output() submitForm = new EventEmitter<Recipe>();

  controls: any
  companyEntries$: Observable<CreInputEntry[]>
  hasCompanies = true

  constructor(
    private companyService: CompanyService,
    private accountService: AccountService,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router,
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.fetchCompanies()

    this.controls = {
      name: new FormControl(this.recipe?.name, Validators.required),
      description: new FormControl(this.recipe?.description, Validators.required),
      color: new FormControl(this.recipe?.color ?? '#ffffff', Validators.required),
      gloss: new FormControl(this.recipe?.gloss ?? 0, Validators.compose([Validators.required, Validators.min(0), Validators.max(100)])),
      sample: new FormControl(this.recipe?.sample, Validators.compose([Validators.required, Validators.min(0)])),
      approbationDate: new FormControl(this.recipe?.approbationDate),
      remark: new FormControl(this.recipe?.remark),
      company: new FormControl({value: this.recipe?.company.id, disabled: !!this.recipe}, Validators.required)
    }
  }

  private fetchCompanies() {
    this.companyEntries$ = this.companyService.all.pipe(
      tap(companies => this.hasCompanies = companies.length > 0),
      map(companies => companies.map(c => new CreInputEntry(c.id, c.name))),
    )
  }

  submit() {
    this.submitForm.emit(this.updatedRecipe)
  }

  get updatedRecipe(): Recipe {
    return {
      ...this.recipe,
      name: this.controls.name.value,
      description: this.controls.description.value,
      color: this.controls.color.value,
      gloss: this.controls.gloss.value,
      sample: this.controls.sample.value,
      approbationDate: this.controls.approbationDate.value,
      remark: this.controls.remark.value,
      company: this.controls.company.value,
    }
  }

  get hasCompanyEditPermission(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_COMPANIES)
  }
}

@Component({
  selector: 'cre-recipe-add',
  templateUrl: 'add.html'
})
export class RecipeAdd extends ErrorHandlingComponent {
  errorHandlers = [{
    filter: error => error.type === `exists-recipe-company-name`,
    messageProducer: error => `Une couleur avec le nom ${error.name} existe déjà pour la bannière ${error.company}`
  }]

  constructor(
    private recipeService: RecipeService,
    private companyService: CompanyService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Nouvelle couleur'
  }

  submit(recipe: Recipe) {
    this.subscribe(
      this.recipeService.save(recipe),
      recipe => this.urlUtils.navigateTo(`/color/edit/${recipe.id}`)
    )
  }
}

@Component({
  selector: 'cre-recipe-edit',
  templateUrl: 'edit.html',
  styleUrls: ['recipes.sass']
})
export class RecipeEdit extends ErrorHandlingComponent {
  @ViewChild(StepTableComponent) stepTable: StepTableComponent
  @ViewChild(RecipeForm) form: RecipeForm

  recipe: Recipe
  groups$ = this.groupService.all
  units$ = new Subject<string>()

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'notfound-recipe-id',
    consumer: _ => this.urlUtils.navigateTo('/color/list')
  }]

  constructor(
    private recipeService: RecipeService,
    private companyService: CompanyService,
    private groupService: GroupService,
    private appState: AppState,
    private alertService: AlertService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)

    this.fetchRecipe()
  }

  private fetchRecipe() {
    const recipeId = this.urlUtils.parseIntUrlParam('id')
    this.subscribe(
      this.recipeService.getById(recipeId),
      recipe => {
        this.recipe = recipe
        this.appState.title = `${recipe.name} (Modifications)`

        if (recipeMixCount(this.recipe) == 0) {
          this.alertService.pushWarning('Il n\'y a aucun mélange dans cette recette')
        }
        if (recipeStepCount(this.recipe) == 0) {
          this.alertService.pushWarning('Il n\'y a aucune étape dans cette recette')
        }
      },
      true,
      1
    )
  }

  changeUnits(unit: string) {
    this.units$.next(unit)
  }

  submit() {
    const recipe = this.form.updatedRecipe
    const steps = this.stepTable.mappedUpdatedSteps

    if (!this.stepsPositionsAreValid(steps)) {
      this.alertService.pushError('Les étapes ne peuvent pas avoir une position inférieure à 1')
      return
    }

    this.subscribeAndNavigate(
      this.recipeService.update(recipe, steps),
      '/color/list'
    )
  }

  delete() {
    this.subscribeAndNavigate(
      this.recipeService.delete(this.recipe.id),
      '/color/list'
    )
  }

  get loggedInUserGroupId(): number {
    return this.appState.authenticatedUser.group?.id
  }

  private stepsPositionsAreValid(steps: Map<number, RecipeStep[]>): boolean {
    return !anyMap(steps, (groupId, steps) => !!steps.find(s => s.position === 0))
  }
}
