import {Injectable} from '@angular/core'
import {ApiService} from '../../shared/service/api.service'
import {convertMixMaterialQuantity, UNIT_MILLILITER} from '../../shared/units'
import {Observable} from 'rxjs'
import {Mix, MixMaterialDto} from '../../shared/model/recipe.model'

@Injectable({
  providedIn: 'root'
})
export class MixService {
  constructor(
    private api: ApiService
  ) {
  }

  get all(): Observable<Mix[]> {
    return this.api.get<Mix[]>('/recipe/mix')
  }

  getById(id: number): Observable<Mix> {
    return this.api.get<Mix>(`/recipe/mix/${id}`)
  }

  saveDto(dto: MixSaveDto): Observable<void> {
    return this.saveWithUnits(
      dto.name,
      dto.recipeId,
      dto.materialTypeId,
      dto.mixMaterials,
    )
  }

  saveWithUnits(name: string, recipeId: number, materialTypeId: number, mixMaterials: MixMaterialDto[]): Observable<void> {
    return this.save(name, recipeId, materialTypeId, this.convertMixMaterialsToMl(mixMaterials))
  }

  save(name: string, recipeId: number, materialTypeId: number, mixMaterials: MixMaterialDto[]): Observable<void> {
    const body = {
      name,
      recipeId,
      materialTypeId,
      mixMaterials: []
    }
    this.appendMixMaterialsToBody(mixMaterials, body)
    return this.api.post('/recipe/mix', body)
  }

  updateDto(dto: MixUpdateDto): Observable<void> {
    return this.updateWithUnits(
      dto.id,
      dto.name,
      dto.materialTypeId,
      dto.mixMaterials
    )
  }

  updateWithUnits(id: number, name: string, materialTypeId: number, mixMaterials: MixMaterialDto[]): Observable<void> {
    return this.update(id, name, materialTypeId, this.convertMixMaterialsToMl(mixMaterials))
  }

  update(id: number, name: string, materialTypeId: number, mixMaterials: MixMaterialDto[]): Observable<void> {
    const body = {
      id,
      name,
      materialTypeId,
      mixMaterials: []
    }

    this.appendMixMaterialsToBody(mixMaterials, body)
    return this.api.put('/recipe/mix', body)
  }

  delete(id: number): Observable<void> {
    return this.api.delete(`/recipe/mix/${id}`)
  }

  private convertMixMaterialsToMl(mixMaterials: MixMaterialDto[]): MixMaterialDto[] {
    return mixMaterials.map(mixMaterial => ({
      ...mixMaterial,
      quantity: convertMixMaterialQuantity(mixMaterial, UNIT_MILLILITER),
      units: UNIT_MILLILITER
    }))
  }

  private appendMixMaterialsToBody(mixMaterials: MixMaterialDto[], body: any) {
    mixMaterials.filter(m => m.materialId != null && m.quantity != null).forEach(m => {
      body.mixMaterials.push({
        materialId: m.materialId,
        quantity: m.quantity,
        position: m.position
      })
    })
  }
}

export interface MixSaveDto {
  name: string
  recipeId: number
  materialTypeId: number
  mixMaterials: MixMaterialDto[]
}

export interface MixUpdateDto {
  id: number
  name: string
  materialTypeId: number
  mixMaterials: MixMaterialDto[]
}

