import {Injectable} from '@angular/core'
import {ApiService} from '../../shared/service/api.service'
import {Observable} from 'rxjs'
import {Recipe, RecipeStep} from '../../shared/model/recipe.model'
import {map} from 'rxjs/operators'

@Injectable({
  providedIn: 'root'
})
export class RecipeService {
  constructor(
    private api: ApiService
  ) {
  }

  get all(): Observable<Recipe[]> {
    return this.api.get<Recipe[]>('/recipe')
  }

  getAllByName(name: string): Observable<Recipe[]> {
    return this.api.get<Recipe[]>(`/recipe?name=${name}`)
  }

  get allByCompany(): Observable<Map<number, Recipe[]>> {
    return this.all.pipe(map(recipes => {
      const map = new Map<number, Recipe[]>()
      recipes.forEach(r => {
        if (!map.has(r.company.id)) {
          map.set(r.company.id, [])
        }
        map.get(r.company.id).push(r)
      })
      return map
    }))
  }

  getById(id: number): Observable<Recipe> {
    return this.api.get<Recipe>(`/recipe/${id}`)
  }

  save(recipe: Recipe): Observable<Recipe> {
    const body = {
      ...recipe,
      companyId: recipe.company
    }

    return this.api.post<Recipe>('/recipe', body)
  }

  update(recipe: Recipe, steps: Map<number, RecipeStep[]>) {
    const body = {
      ...recipe,
      steps: []
    }

    steps.forEach((groupSteps, groupId) => {
      const mappedGroupSteps = groupSteps.map(s => {
        return {message: s.message, position: s.position}
      })
      body.steps.push({groupId, steps: mappedGroupSteps})
    })

    return this.api.put<Recipe>('/recipe', body)
  }

  updateExplorerModifications(id: number, notes: Map<number, string>, mixesLocationChange: Map<number, string>): Observable<void> {
    const body = {
      recipeId: id,
      notes: [],
      mixesLocation: []
    }

    notes.forEach((content, groupId) => {
      body.notes.push({groupId, content})
    })

    mixesLocationChange.forEach((location, mixId) => {
      body.mixesLocation.push({mixId, location})
    })

    return this.api.put<void>('/recipe/public', body)
  }

  delete(id: number): Observable<void> {
    return this.api.delete<void>(`/recipe/${id}`)
  }
}
