import {
  AfterViewInit,
  ChangeDetectorRef,
  Component,
  ContentChild,
  Directive,
  ElementRef,
  EventEmitter,
  Input,
  OnDestroy,
  OnInit,
  Output,
  TemplateRef,
  ViewChild,
  ViewEncapsulation
} from '@angular/core'
import {AbstractControl, FormControl, ValidationErrors, ValidatorFn, Validators} from '@angular/forms'
import {COMMA, ENTER} from '@angular/cdk/keycodes'
import {isObservable, Observable, Subject} from 'rxjs'
import {map, takeUntil} from 'rxjs/operators'
import {MatChipInputEvent} from '@angular/material/chips'
import {MatAutocomplete, MatAutocompleteSelectedEvent} from '@angular/material/autocomplete'

@Directive()
abstract class _CreInputBase {
  @Input() control: FormControl | null
  @Input() label: string
  @Input() value
  @Input() disabled = false
}

@Component({
  selector: 'cre-input',
  templateUrl: 'input.html',
  encapsulation: ViewEncapsulation.None,
  styleUrls: ['input.sass']
})
export class CreInputComponent extends _CreInputBase implements AfterViewInit {
  @Input() type = 'text'
  @Input() icon: string
  @Input() iconTitle: string | null
  @Input() required = true
  @Input() autocomplete = true
  @Input() placeholder: string | null
  @Input() step = 1
  @Input() iconColor: string = 'black'
  @Input() hint: string | null

  @Output() valueChange = new EventEmitter<any>()

  @ViewChild('input') input: any
  @ContentChild(TemplateRef) errors: TemplateRef<any>

  fieldRequired = false

  constructor(
    private cdRef: ChangeDetectorRef
  ) {
    super()
  }

  ngAfterViewInit() {
    const element = this.input.nativeElement
    element.type = this.type
    element.step = this.step.toString()
    element.placeholder = this.placeholder
    element.autocomplete = this.autocomplete ? 'on' : 'off'

    this.fieldRequired = this.control ? this.control.validator && this.control.validator({} as AbstractControl)?.required : this.required

    this.cdRef.detectChanges()
  }
}

@Component({
  selector: 'cre-textarea',
  templateUrl: 'textarea.html',
  encapsulation: ViewEncapsulation.None
})
export class CreTextareaComponent {
  @Input() label: string
  @Input() control: FormControl
  @Input() cols = 40
  @Input() rows = 3
  @Input() placeholder: string | null
}

@Component({
  selector: 'cre-autocomplete-input',
  templateUrl: 'autocomplete.html',
  encapsulation: ViewEncapsulation.None
})
export class CreAutocompleteInputComponent {
  @Input() control: AbstractControl | null
  @Input() label: string
  @Input() icon: string
  @Input() required = true
  @Input() entries: Observable<string[]>
  @Input() value

  @Output() valueChange = new EventEmitter<any>()

  @ContentChild(TemplateRef) errors: TemplateRef<any>
}

@Component({
  selector: 'cre-chip-input',
  templateUrl: 'chips.html',
  encapsulation: ViewEncapsulation.None
})
export class CreChipInputComponent implements OnInit {
  @Input() control: AbstractControl
  @Input() label: string
  @Input() icon: string
  @Input() required = true

  inputControl = new FormControl()

  readonly separatorKeysCodes = [ENTER, COMMA]

  selectedValues = []

  ngOnInit(): void {
    if (this.control.value) {
      this.selectedValues = this.control.value
    }
  }

  add(event: MatChipInputEvent): void {
    const input = event.input
    const value = event.value

    if ((value || '').trim()) {
      this.selectedValues.push(value.trim())
    }

    if (input) {
      input.value = ''
    }

    this.inputControl.setValue(null)
    this.control.setValue(this.selectedValues)
  }

  remove(value): void {
    const index = this.selectedValues.indexOf(value)

    if (index >= 0) {
      this.selectedValues.splice(index, 1)
      this.control.setValue(this.selectedValues)
    }
  }
}

@Component({
  selector: 'cre-combo-box',
  templateUrl: 'combo-box.html',
  encapsulation: ViewEncapsulation.None
})
export class CreComboBoxComponent {
  @Input() control: AbstractControl
  @Input() label: string
  @Input() icon: string
  @Input() required = true

  @ContentChild(TemplateRef) errors: TemplateRef<any>

  internalControl: FormControl
  filteredEntries: CreInputEntry[]
  validValue = false

  private _destroy$ = new Subject<boolean>()
  private _entries: CreInputEntry[]
  private _controlsInitialized = false

  @Input()
  set entries(entries: Observable<CreInputEntry[]> | CreInputEntry[]) {
    if (isObservable(entries)) {
      (entries as Observable<CreInputEntry[]>).pipe(takeUntil(this._destroy$))
        .subscribe({
          next: entries => {
            this.initControls(entries)
          }
        })
    } else {
      this.initControls((entries as CreInputEntry[]))
    }
  }

  reloadEntries() {
    this.filteredEntries = this.filterEntries(this.internalControl.value)
  }

  private initControls(entries) {
    this._entries = entries
    if (this._controlsInitialized) {
      return
    }

    this.internalControl = new FormControl({
      value: null,
      disabled: false
    }, Validators.compose([this.control.validator, this.valueValidator()]))
    this.internalControl.valueChanges
      .pipe(takeUntil(this._destroy$))
      .subscribe({
        next: value => {
          if (this.internalControl.valid) {
            this.control.setValue(this.findEntryByValue(value).key)
          } else {
            this.control.setValue(null)
          }

          this.filteredEntries = this.filterEntries(value)
        }
      })

    if (this.control.value) {
      this.internalControl.setValue(this.findEntryByKey(this.control.value)?.value)
    }

    if (this.control.disabled) {
      this.internalControl.disable()
    }

    this.reloadEntries()
    this._controlsInitialized = true
  }

  private filterEntries(value: string): CreInputEntry[] {
    if (!value) {
      return this._entries
    }

    const valueLowerCase = value.toLowerCase()
    return this._entries.filter(entry => {
      if (entry.display) {
        return entry.display.toLowerCase().includes(valueLowerCase)
      } else {
        return entry.value.toLowerCase().includes(valueLowerCase)
      }
    })
  }

  private findEntryByKey(key: any): CreInputEntry | null {
    const found = this._entries.filter(e => e.key === key)
    if (found.length <= 0) {
      return null
    }
    return found[0]
  }

  private findEntryByValue(value: any): CreInputEntry | null {
    const found = this._entries.filter(e => e.value === value)
    if (found.length <= 0) {
      return null
    }
    return found[0]
  }

  private existsEntryByValue(value: any): boolean {
    return this._entries && this._entries.filter(o => o.value === value).length > 0
  }

  private valueValidator(): ValidatorFn {
    return (control: AbstractControl): ValidationErrors | null => {
      const valid = this.existsEntryByValue(control.value)
      return valid ? null : {invalidValue: {value: control.value}}
    }
  }
}

@Component({
  selector: 'cre-chip-combo-box',
  templateUrl: 'chips-combo-box.html',
  encapsulation: ViewEncapsulation.None
})
export class CreChipComboBoxComponent extends CreChipInputComponent implements OnDestroy {
  @Input() entries: Observable<CreInputEntry[]>

  @ContentChild(TemplateRef) errors: TemplateRef<any>
  @ViewChild('chipInput') chipInput: ElementRef<HTMLInputElement>
  @ViewChild('auto') matAutocomplete: MatAutocomplete

  filteredEntries: Observable<CreInputEntry[]>

  private _entries: CreInputEntry[]
  private _destroy$ = new Subject()

  ngOnInit() {
    super.ngOnInit()

    this.entries.pipe(takeUntil(this._destroy$))
      .subscribe({
        next: entries => this._entries = entries
      })

    this.filteredEntries = this.inputControl.valueChanges.pipe(
      map((query: string | null) => query ? this._filter(query) : this._entries.slice())
    )
  }

  ngOnDestroy(): void {
    this._destroy$.complete()
  }

  selected(event: MatAutocompleteSelectedEvent) {
    this.selectedValues.push(this.findValueByKey(event.option.value))
    this.chipInput.nativeElement.value = ''
    this.inputControl.setValue(null)
    this.control.setValue(this.selectedValues)
  }

  get empty(): boolean {
    return this.selectedValues.length <= 0
  }

  private _filter(query: string): CreInputEntry[] {
    const filterValue = query.toString().toLowerCase()
    return this._entries.filter(e => e.value.toString().toLowerCase().indexOf(filterValue) === 0)
  }

  private findValueByKey(key: any): any {
    return this._entries.filter(e => e.key === key)[0].value
  }
}

@Component({
  selector: 'cre-checkbox-input',
  templateUrl: 'checkbox.html'
})
export class CreCheckboxInputComponent implements OnInit {
  @Input() label: string
  @Input() control: AbstractControl
  @Input() checked: boolean

  @Output() checkedChange = new EventEmitter<boolean>()

  ngOnInit(): void {
    this.control?.setValue(this.control.value === 'true')
  }
}

@Component({
  selector: 'cre-file-input',
  templateUrl: 'file.html'
})
export class CreFileInputComponent implements OnInit {
  @Input() label: string
  @Input() icon: string
  @Input() accept = ''
  @Input() control: AbstractControl | null

  @Output() selection = new EventEmitter<File>()
  @Output() invalidFormat = new EventEmitter<void>()

  private acceptedMediaTypes: string[]

  ngOnInit(): void {
    this.acceptedMediaTypes = this.accept.split(',')
  }

  selectedFile: File | null
  selectedFileName: string

  onFileSelected(event: any) {
    this.selectedFile = event.target.files[0]
    if (this.acceptedMediaTypes.indexOf(this.selectedFile.type) >= 0) {
      this.selectedFileName = this.selectedFile.name
      this.control?.setValue(this.selectedFile)
      this.control?.markAsDirty()
      this.selection.emit(this.selectedFile)
    } else {
      this.invalidFormat.emit()
    }
  }
}

@Component({
  selector: 'cre-period-input',
  templateUrl: 'period.html',
  styleUrls: ['input.sass'],
  encapsulation: ViewEncapsulation.None
})
export class CrePeriodInputComponent implements OnInit {
  @Input() control: AbstractControl
  @Input() label: string
  @Input() hint: string | null

  readonly periodTypes = [
    {value: 'D', label: 'Jours'},
    {value: 'M', label: 'Mois'},
    {value: 'Y', label: 'Années'}
  ]
  readonly inputControl = new FormControl(null, Validators.compose([Validators.required, Validators.min(1), Validators.pattern(/^[0-9]\d*$/)]))
  readonly selectControl = new FormControl()

  ngOnInit(): void {
    this.setValuesFromPeriod(this.control.value as string)
  }

  inputControlUpdate(value: string) {
    this.updateControl(value, this.selectControl.value)
  }

  selectControlUpdate(event: any) {
    this.updateControl(this.inputControl.value, event.value)
  }

  private updateControl(inputValue: string, selectValue: string) {
    this.control.setValue('P' + inputValue + selectValue)
    this.control.markAsDirty()
  }

  private setValuesFromPeriod(period: string) {
    if (!period) {
      return
    }

    const periodTypeChar = period.slice(-1)
    period = period.slice(1, -1)
    this.selectControl.setValue(periodTypeChar)
    this.inputControl.setValue(period)
  }
}

@Component({
  selector: 'cre-slider-input',
  templateUrl: 'slider.html'
})
export class CreSliderInputComponent {
  @Input() control: FormControl
  @Input() label: string
  @Input() min: number
  @Input() max: number
  @Input() step = 1
  @Input() percents = false
  @Input() thumbLabel = true

  formatValueForDisplay(value: number): string {
    return this.percents ? `${value}%` : value.toString()
  }
}

@Component({
  selector: 'cre-select',
  templateUrl: 'select.html'
})
export class CreSelectComponent extends _CreInputBase {
  @Input() entries: CreInputEntry[] | Observable<CreInputEntry[]>

  get entriesAreObservable(): boolean {
    return isObservable(this.entries)
  }

  get arrayEntries(): CreInputEntry[] {
    return this.entries as CreInputEntry[]
  }

  get observableEntries(): Observable<CreInputEntry[]> {
    return this.entries as Observable<CreInputEntry[]>
  }
}

export class CreInputEntry {
  constructor(
    public key: any,
    public value: any,
    public display?: any
  ) {
  }
}

export function chipListRequired(): ValidatorFn {
  return (control: AbstractControl): ValidationErrors | null => {
    return !control.value || control.value.length <= 0 ? {required: true} : null
  }
}
