import {take, takeUntil} from 'rxjs/operators'
import { OnDestroy, OnInit, Directive } from '@angular/core'
import {Observable, Subject} from 'rxjs'
import {ActivatedRoute, Router} from '@angular/router'
import {UrlUtils} from '../utils/url.utils'
import {ErrorHandler, ErrorHandlerComponent, ErrorService} from '../service/error.service'
import {globalLoadingWheel} from './loading-wheel/loading-wheel.component'

@Directive()
export abstract class SubscribingComponent implements OnInit, OnDestroy {
  protected subscribers$ = []
  protected destroy$ = new Subject<boolean>()

  loading = false

  protected constructor(
    protected errorService: ErrorService,
    protected activatedRoute: ActivatedRoute,
    protected router: Router,
    protected urlUtils = new UrlUtils(activatedRoute, router)
  ) {
  }

  subscribe<T>(observable: Observable<T>, resultConsumer: (T) => void, showWheel = false, take_count = -1) {
    if (take_count >= 0) {
      observable.pipe(take(take_count), takeUntil(this.destroy$))
    } else {
      observable.pipe(takeUntil(this.destroy$))
    }

    this.showLoadingWheel(showWheel)
    this.subscribers$.push(observable.subscribe({
      next: t => {
        resultConsumer(t)
        this.hideLoadingWheel(showWheel)
      },
      error: err => {
        this.errorService.handleError(err)
        this.hideLoadingWheel(showWheel)
      }
    }))
  }

  subscribeAndNavigate(observable: Observable<any>, route: string, showWheel = true) {
    this.subscribe(
      observable,
      () => this.urlUtils.navigateTo(route),
      showWheel,
      1
    )
  }

  subscribeEntityById<T>(service: any, id: number, resultConsumer: (T) => void, showWheel = true) {
    this.showLoadingWheel(showWheel)
    this.subscribers$.push(service.getById(id)
      .pipe(take(1), takeUntil(this.destroy$))
      .subscribe({
        next: e => {
          resultConsumer(e)
          this.hideLoadingWheel(showWheel)
        },
        error: err => {
          this.hideLoadingWheel(showWheel)
          this.errorService.handleError(err)
        }
      }))
  }

  ngOnInit(): void {
  }

  ngOnDestroy(): void {
    this.destroy$.next(true)
    this.destroy$.complete()
  }

  protected showLoadingWheel(shouldShowWheel) {
    if (shouldShowWheel) {
      this.loading = true
      globalLoadingWheel.show()
    }
  }

  protected hideLoadingWheel(shouldShowWheel) {
    if (shouldShowWheel) {
      this.loading = false
      globalLoadingWheel.hide()
    }
  }
}

@Directive()
export abstract class ErrorHandlingComponent extends SubscribingComponent implements ErrorHandlerComponent {
  errorHandlers: ErrorHandler[] = []

  protected constructor(
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
    this.errorService.activeErrorHandler = this
  }
}
