import {
  AfterContentInit,
  Component,
  ContentChildren,
  Directive,
  EventEmitter,
  HostBinding,
  Input,
  Output,
  QueryList,
  ViewChild,
  ViewEncapsulation
} from '@angular/core'
import {MatColumnDef, MatHeaderRowDef, MatRowDef, MatTable, MatTableDataSource} from '@angular/material/table'

@Directive({
  selector: '[creInteractiveCell]'
})
export class CreInteractiveCell implements AfterContentInit {
  @Input('creInteractiveCell') index: number

  @HostBinding() hidden = true

  private _selectedIndex = 0
  private _hoverIndex = 0

  ngAfterContentInit(): void {
    this.hidden = this.isHidden
  }

  set hoverIndex(index: number) {
    this._hoverIndex = index
    this.hidden = this.isHidden
  }

  set selectedIndex(index: number) {
    this._selectedIndex = index
    this.hidden = this.isHidden
  }

  get isHidden(): boolean {
    return this._hoverIndex !== this.index && this._selectedIndex !== this.index
  }
}

@Component({
  selector: 'cre-table',
  templateUrl: 'table.html',
  styleUrls: ['table.sass'],
  encapsulation: ViewEncapsulation.None
})
export class CreTable<T> implements AfterContentInit {
  @ContentChildren(MatHeaderRowDef) headerRowDefs: QueryList<MatHeaderRowDef>
  @ContentChildren(MatRowDef) rowDefs: QueryList<MatRowDef<T>>
  @ContentChildren(MatColumnDef) columnDefs: QueryList<MatColumnDef>

  @ContentChildren(CreInteractiveCell, {descendants: true}) interactiveCells: QueryList<CreInteractiveCell>

  @ViewChild(MatTable, {static: true}) table: MatTable<T>

  @Input() columns: string[]
  @Input() interactive = true
  @Input() filterPredicate: (t: T, filter: string) => boolean = () => true
  @Input() sortingDataAccessor: (t: T, header: string) => string | number

  @Input() set filter(filter: string) {
    if (this.dataSource) {
      this.dataSource.filter = filter
    }
  }

  @Input() set data(data: T[]) {
    this.setupDataSource(data)
  }

  selectedIndex = 0

  dataSource: MatTableDataSource<T>

  ngAfterContentInit(): void {
    this.columnDefs.forEach(columnDef => this.table.addColumnDef(columnDef))
    this.rowDefs.forEach(rowDef => this.table.addRowDef(rowDef))
    this.headerRowDefs.forEach(headerRowDef => this.table.addHeaderRowDef(headerRowDef))
  }

  private setupDataSource(data: T[]) {
    this.dataSource = new MatTableDataSource<T>(data)

    if (this.filterPredicate) {
      this.dataSource.filterPredicate = (t, filter) => this.filterPredicate(t, filter)
    }
  }

  onRowHover(index: number) {
    if (this.interactive) {
      this.interactiveCells.forEach(cell => cell.hoverIndex = index)
    }
  }

  onRowSelected(index: number) {
    if (this.interactive) {
      this.selectedIndex = index
      this.interactiveCells.forEach(cell => cell.selectedIndex = index)
    }
  }

  renderRows() {
    this.table.renderRows()
  }
}

@Component({
  selector: 'cre-table-position-buttons',
  templateUrl: 'position-buttons.html'
})
export class CrePositionButtons {
  @Input() position = 0
  @Input() min = 0
  @Input() max: number
  @Input() hidden = false
  @Input() disableDecreaseButton = false
  @Input() disableIncreaseButton = false

  @Output() positionChange = new EventEmitter<number>()

  increasePosition() {
    this.position += 1
    this.positionChange.emit(this.position)
  }

  decreasePosition() {
    this.position -= 1
    this.positionChange.emit(this.position)
  }
}
