import {Material} from './material.model'
import {Company} from './company.model'
import {Group} from './user'
import {UNIT_MILLILITER} from "../units";

export class Recipe {
  public id: number
  public name: string
  public description: string
  public color: string
  public gloss: number
  public sample: number
  public approbationDate: string
  public remark: string
  public company: Company
  public mixes: Mix[]
  public approbationExpired: boolean
  public groupsInformation: RecipeGroupInformation[]
  public imagesUrls: string[]
}

export class RecipeGroupInformation {
  constructor(
    public id: number,
    public group: Group,
    public note: string,
    public steps: RecipeStep[]
  ) {
  }
}

export class Mix {
  constructor(
    public id: number,
    public mixType: MixType,
    public mixMaterials: MixMaterial[],
    public location: string,
  ) {
  }
}

export class MixMaterial {
  constructor(
    public id: number,
    public material: Material,
    public quantity: number,
    public position: number
  ) {
  }
}

export class MixMaterialDto {
  constructor(
    public materialId: number,
    public quantity: number,
    public isPercents: boolean,
    public position: number,
    public units: string
  ) {
  }
}

class MixType {
  constructor(
    public id: number,
    public name: string,
    public material: Material
  ) {
  }
}

export class RecipeStep {
  constructor(
    public id: number,
    public message: string,
    public position: number
  ) {
  }
}

export function recipeMixCount(recipe: Recipe): number {
  return recipe.mixes.length
}

export function recipeStepCount(recipe: Recipe): number {
  return recipe.groupsInformation.length > 0 ? recipe.groupsInformation.flatMap(i => i.steps).length : 0
}

export function recipeNoteForGroupId(recipe: Recipe, groupId: number): string | null {
  const groupInformation = recipe.groupsInformation.find(i => i.group.id === groupId)
  return groupInformation ? groupInformation.note : null
}

export function recipeStepsForGroupId(recipe: Recipe, groupId: number): RecipeStep[] {
  const groupInformation = recipe.groupsInformation.find(i => i.group.id === groupId)
  return groupInformation ? sortRecipeSteps(groupInformation.steps) : []
}

export function sortRecipeSteps(steps: RecipeStep[]): RecipeStep[] {
  return steps.sort((a, b) => a.position - b.position)
}

export function mixMaterialsToMixMaterialsDto(mix: Mix): MixMaterialDto[] {
  return sortMixMaterialsDto(mix.mixMaterials.map(m => new MixMaterialDto(
    m.material.id,
    m.quantity,
    m.material.materialType.usePercentages,
    m.position,
    UNIT_MILLILITER
  )))
}

export function sortMixMaterialsDto(mixMaterials: MixMaterialDto[]): MixMaterialDto[] {
  return mixMaterials.sort((a, b) => a.position - b.position)
}

export function getRecipeLuma(recipe: Recipe): number {
  // https://stackoverflow.com/questions/12043187/how-to-check-if-hex-color-is-too-black
  const c = recipe.color.substring(1)      // strip #
  const rgb = parseInt(c, 16)   // convert rrggbb to decimal
  const r = (rgb >> 16) & 0xff  // extract red
  const g = (rgb >> 8) & 0xff  // extract green
  const b = (rgb >> 0) & 0xff  // extract blue

  return 0.2126 * r + 0.7152 * g + 0.0722 * b // per ITU-R BT.709
}

export function recipeMatchesFilter(recipe: Recipe, filter: string): boolean {
  const recipeStr = recipe.company.name + recipe.name + recipe.description + recipe.sample
  return recipeStr.toLowerCase().indexOf(filter.toLowerCase()) >= 0
}
