import {Directive, Injectable, OnDestroy} from '@angular/core';
import {interval, Subject} from 'rxjs'
import {takeUntil} from 'rxjs/operators'

@Injectable({
  providedIn: 'root'
})
export class AlertService {
  private alertQueue: Alert[] = []
  private activeHandler: AlertHandlerComponent

  pushSuccess(message: String) {
    this.enqueue(AlertType.Success, message)
  }

  pushWarning(message: String) {
    this.enqueue(AlertType.Warning, message)
  }

  pushError(message: String) {
    this.enqueue(AlertType.Error, message)
  }

  set activeAlertHandlerComponent(handler: AlertHandlerComponent) {
    this.activeHandler = handler
  }

  private enqueue(type: AlertType, message: String) {
    this.alertQueue.unshift({type, message})
  }

  dequeue(): Alert {
    return this.alertQueue.pop()
  }
}

/**
 * An alert handler component is a component that will show the alerts pushed by the alert system to the user.
 */
@Directive()
// eslint-disable-next-line @angular-eslint/directive-class-suffix
export abstract class AlertHandlerComponent implements OnDestroy {
  protected static readonly DEFAULT_ALERT_BUFFER_SIZE = 3
  protected static readonly DEFAULT_ALERT_DURATION = 5

  alertBuffer = new Array<{ alert: Alert, time: number }>()
  protected alertBufferSize: number = AlertHandlerComponent.DEFAULT_ALERT_BUFFER_SIZE
  protected alertDuration: number = AlertHandlerComponent.DEFAULT_ALERT_DURATION
  protected alertDurationCounter = 0

  private destroy$ = new Subject<boolean>()

  protected constructor(
    protected alertService: AlertService
  ) {
    alertService.activeAlertHandlerComponent = this

    interval(1000)
      .pipe(takeUntil(this.destroy$))
      .subscribe({
        next: () => {
          this.alertDurationCounter++
          if (!this.isBufferEmpty) {
            this.alertBuffer
              .filter(a => a.time + this.alertDuration < this.alertDurationCounter)
              .map(a => this.alertBuffer.indexOf(a))
              .forEach(i => this.alertBuffer.splice(i, 1))
          }
          if (!this.isBufferFull) {
            this.pullAlert()
          }
        }
      })
  }

  ngOnDestroy(): void {
    this.destroy$.next(true)
    this.destroy$.complete()
  }

  pullAlert() {
    const nextAlert = this.alertService.dequeue()
    if (nextAlert) {
      this.alertBuffer.unshift({alert: nextAlert, time: this.alertDurationCounter})
    }
  }

  get isBufferEmpty(): boolean {
    return this.alertBuffer.length == 0
  }

  get isBufferFull(): boolean {
    return this.alertBuffer.length >= this.alertBufferSize
  }
}

/**
 * An alert is a message that will be shown to the user.
 *
 * An alert can be a success alert, a warning alert of an error alert.
 */
class Alert {
  constructor(
    public type: AlertType,
    public message: String
  ) {
  }
}

export enum AlertType {
  Success,
  Warning,
  Error
}
