import {Injectable} from '@angular/core'
import {Config, filterConfigKeyControlMap, filterImageConfigKeyControlMap, mapToConfigKeyContentArray} from '../model/config.model'
import {Observable} from 'rxjs'
import {ApiService} from './api.service'
import {AbstractControl, FormGroup} from '@angular/forms'
import {transformMap} from '../utils/map.utils'

@Injectable({
  providedIn: 'root'
})
export class ConfigService {
  constructor(
    private api: ApiService
  ) {
  }

  get all(): Observable<Config[]> {
    return this.api.get<Config[]>('/config')
  }

  get(key: string): Observable<Config> {
    return this.api.get<Config>(`/config/${key}`)
  }

  setFromForm(form: FormGroup): Observable<void> {
    const map = new Map<string, AbstractControl>()
    for (let key in form.controls) {
      map.set(key, form.controls[key])
    }
    return this.set(map);
  }

  set(configs: Map<string, AbstractControl>): Observable<void> {
    const body = mapToConfigKeyContentArray(filterConfigKeyControlMap(configs))
    const imageConfigs = filterImageConfigKeyControlMap(configs)

    this.setImages(imageConfigs)
    return this.api.put<void>('/config', body)
  }

  setImage(key: string, image: File): Observable<void> {
    const path = key == Config.INSTANCE_ICON_SET ? 'icon' : 'logo';

    const body = new FormData()
    body.append(path, image)

    return this.api.put<void>(`/config/${path}`, body)
  }

  restart(): Observable<void> {
    return this.api.post<void>('/config/restart')
  }

  private setImages(configs: Map<string, AbstractControl>) {
    const subscriptions = this.getImageConfigsSubscriptions(configs)
    while (subscriptions.length > 0) {
      const subscription = subscriptions.pop().subscribe({
        next: () => subscription.unsubscribe()
      })
    }
  }

  private getImageConfigsSubscriptions(configs: Map<string, AbstractControl>): Observable<void>[] {
    return transformMap(configs, (key, control) => {
      return this.setImage(key, control.value)
    })
  }
}
