import {Component, Input} from '@angular/core'
import {TouchUpKit} from '../../shared/model/touch-up-kit.model'
import {formatDate, openTouchUpKit, reduceDashes} from '../../shared/utils/utils'
import {ErrorHandlingComponent} from '../../shared/components/subscribing.component'
import {TouchUpKitService} from '../service/touch-up-kit.service'
import {AccountService} from '../../accounts/services/account.service'
import {ErrorService} from '../../shared/service/error.service'
import {ActivatedRoute, Router} from '@angular/router'
import {Permission} from '../../shared/model/user'
import {RecipeService} from '../../recipes/services/recipe.service'
import {AppState} from '../../shared/app-state'
import {map} from 'rxjs/operators'
import {LocalDate, Period} from '@js-joda/core'
import {ConfigService} from '../../shared/service/config.service'
import {Config} from '../../shared/model/config.model'

@Component({
  selector: 'touchupkit-banner',
  templateUrl: 'banner.html',
  styles: [
    'p { margin-bottom: 0 }'
  ]
})
export class TouchUpKitBanner {
  @Input() touchUpKit: TouchUpKit

  get shippingDate(): string {
    return formatDate(this.touchUpKit.shippingDate)
  }

  get completionDate(): string {
    return formatDate(this.touchUpKit.completionDate)
  }

  get material(): string {
    return reduceDashes(this.touchUpKit.material)
  }
}

@Component({
  selector: 'touchupkit-list',
  templateUrl: './list.html'
})
export class TouchUpKitList extends ErrorHandlingComponent {
  touchUpKits$ = this.touchUpKitService.all
  completedTouchUpKits$ = this.touchUpKits$.pipe(
    map(kits => kits.filter(k => k.completed))
  )
  uncompletedTouchUpKits$ = this.touchUpKits$.pipe(
    map(kits => kits.filter(k => !k.completed))
  )
  expirationPeriod: Period | null

  columns = ['project', 'buggy', 'company', 'shippingDate', 'pdfButton', 'detailsButton', 'editButton']
  completedColumns = ['project', 'buggy', 'company', 'shippingDate', 'completionDate', 'expirationDate', 'pdfButton', 'detailsButton']

  constructor(
    private touchUpKitService: TouchUpKitService,
    private accountService: AccountService,
    private configService: ConfigService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Kits de retouche'
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribe(
      this.configService.get(Config.TOUCH_UP_KIT_EXPIRATION),
      config => this.expirationPeriod = Period.parse(config.content)
    )
  }

  openTouchUpKitPdf(touchUpKit: TouchUpKit) {
    openTouchUpKit(touchUpKit)
  }

  shippingDate(touchUpKit: TouchUpKit): string {
    return formatDate(touchUpKit.shippingDate)
  }

  completionDate(touchUpKit: TouchUpKit): string {
    return formatDate(touchUpKit.completionDate)
  }

  expirationDate(touchUpKit: TouchUpKit): string {
    const completionDate = LocalDate.parse(touchUpKit.completionDate)
    return formatDate(completionDate.plus(this.expirationPeriod).toString())
  }

  get canEditTouchUpKits(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_TOUCH_UP_KITS)
  }
}

@Component({
  selector: 'touchupkit-details',
  templateUrl: 'details.html'
})
export class TouchUpKitDetails extends ErrorHandlingComponent {
  touchUpKit: TouchUpKit | null
  contentTableCols = ['name', 'description', 'quantity', 'ready']

  constructor(
    private touchUpKitService: TouchUpKitService,
    private recipeService: RecipeService,
    private accountService: AccountService,
    private appState: AppState,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribeEntityById(
      this.touchUpKitService,
      this.urlUtils.parseIntUrlParam('id'),
      t => {
        this.appState.title = `${t.project} - ${t.buggy}`
        this.touchUpKit = t
      }
    )
  }

  openPdf() {
    openTouchUpKit(this.touchUpKit)
  }

  save() {
    this.subscribeAndNavigate(
      this.touchUpKitService.update(this.touchUpKit),
      `/misc/touch-up-kit/details/${this.touchUpKit.id}`
    )
  }

  complete() {
    this.subscribeAndNavigate(
      this.touchUpKitService.complete(this.touchUpKit),
      '/misc/touch-up-kit'
    )
  }

  get canEditTouchUpKits(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_TOUCH_UP_KITS)
  }
}

@Component({
  selector: 'touchupkit-add',
  templateUrl: 'add.html'
})
export class TouchUpKitAdd extends ErrorHandlingComponent {
  constructor(
    private touchUpKitService: TouchUpKitService,
    private appState: AppState,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router,
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Nouveau kit de retouche'
  }

  submit(touchUpKit) {
    this.subscribeAndNavigate(
      this.touchUpKitService.save(touchUpKit),
      '/misc/touch-up-kit/list'
    )
  }
}

@Component({
  selector: 'touchupkit-edit',
  templateUrl: 'edit.html'
})
export class TouchUpKitEdit extends ErrorHandlingComponent {
  touchUpKit: TouchUpKit | null

  constructor(
    private touchUpKitService: TouchUpKitService,
    private appState: AppState,
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribeEntityById(
      this.touchUpKitService,
      this.urlUtils.parseIntUrlParam('id'),
      touchUpKit => {
        this.appState.title = `${touchUpKit.project} - ${touchUpKit.buggy} (Modifications)`
        this.touchUpKit = touchUpKit
      }
    )
  }

  submit(touchUpKit) {
    this.subscribeAndNavigate(
      this.touchUpKitService.update(touchUpKit),
      '/misc/touch-up-kit/list'
    )
  }

  delete() {
    this.subscribeAndNavigate(
      this.touchUpKitService.delete(this.touchUpKit.id),
      '/misc/touch-up-kit/list'
    )
  }
}
